import os
import re
import sys

__all__ = ["tag", "template", "write"]

class ParseError(Exception):
    pass

# Parameters that the caller may set to override the defaults
tag = "redhat:option"
if __name__ == "__main__":
    template = sys.argv[0]
else:
    template = __file__
template = os.path.join(
    os.path.split(os.path.realpath(template))[0], "server.xml.in")

# Function to write a config file
def write(fp, values):
    """Write a server.xml file for a Tomcat server."""
    tag_re = re.compile(r'^(\s*)<(/)?%s((\s+\w+\s*="[^"]*")*\s*)>\s*$' % tag)
    attr_re = re.compile(r'^(\w+)="([^"]*)"$')
    stack = []
    for line in open(template, "r").xreadlines():
        m = tag_re.search(line)
        if not m:
            if not stack or stack[-1] != (0, 1):
                fp.write(line)
            continue
        leadspace, preslash, attrs = m.group(1, 2, 3)
        if not preslash:
            # start tag
            delete = enabled = None
            attrs = attrs.strip().split()
            for attr in attrs:
                key, value = attr_re.search(attr).group(1, 2)
                if key == "else":
                    delete = {"delete": 1}.get(value, 0)
                else:
                    if enabled is not None:
                        raise ParseError, "option has multiple conditions"
                    enabled = values.get(key, None) == value
            if enabled is None:
                raise ParseError, "option has no conditions"
            if not enabled and not delete:
                fp.write(leadspace + "<!--\n")
            stack.append((enabled, delete))
        else:
            # end tag
            enabled, delete = stack.pop()
            if not enabled and not delete:
                fp.write(leadspace + "-->\n")
            

if __name__ == "__main__":
    # Command-line templater
    values = {}
    value_re = re.compile(r"^--(\w+)=(\w+)$")
    for arg in sys.argv[1:]:
        key, value = value_re.search(arg).group(1, 2)
        values[key] = value
    write(sys.stdout, values)
