import os
import sys
import time

__all__ = ["debug", "info", "warn", "error", "stream"]

# Parameters that the caller may set to override the defaults
stream = sys.stdout

# User-callable functions
def error(msg):
    log(3, msg)

def warn(msg):
    log(2, msg)

def info(msg):
    log(1, msg)

def debug(msg):
    log(0, msg)

# Nuts and bolts
levels = ["DEBUG", "INFO", "WARN", "ERROR"]
loglevel = levels.index(os.environ.get("LOGLEVEL", "INFO").upper())

def log(level, msg):
    if level >= loglevel:
        msg = "[%s] %s" % (levels[level], msg)
        if not stream.isatty():
            msg = time.strftime("%H:%M:%S %d-%m-%Y") + " " + msg
        stream.write(msg + "\n")
