import os
import smtplib
import SocketServer

class xSMTPRequestHandler(SocketServer.StreamRequestHandler):
    """SMTP server that logs transactions.

    This is not a complete implementation of RFC 821 by any means;
    just enough is implemented in order to coerce the client into
    completing the transaction.  A non-standard command, '####',
    returns a dump of the previous transaction."""

    def handle(self):
        self.log = []
        self.writeline("220 localhost Service ready")
        while 1:
            line = self.readline()
            command = line.strip().split(" ", 1)[0].upper()
            if command in ("HELO", "MAIL", "RCPT"):
                self.writeline("250 Requested mail action okay, completed")
            elif command == "DATA":
                self.writeline("354 Start mail input; end with <CRLF>.<CRLF>")
                while self.readline().strip() != ".":
                    pass
                self.writeline("250 Requested mail action okay, completed")
            elif command == "QUIT":
                self.writeline("221 localhost Service closing channel")
                self.server.log = self.log
                break
            elif command == "####":
                self.wfile.write("999-" +
                                 "999-".join(self.server.log[:-1]) +
                                 "999 " + self.server.log[-1])
            else:
                self.writeline("502 Command not implemented")

    def readline(self):
        line = self.rfile.readline()
        self.log.append(line)
        return line

    def writeline(self, line):
        line += "\r\n"
        self.log.append(line)
        self.wfile.write(line)

class xSMTPDumpError(smtplib.SMTPResponseException):
    """The server didn't dump the transaction as requested."""

class xSMTPClient(smtplib.SMTP):
    """SMTP client with transaction-dumping support."""
    
    def dump(self):
        code, msg = self.docmd("####")
        if code != 999:
            raise xSMTPDumpError(code, msg)
        return msg

class ChildDiedError(Exception):
    """The SMTP server died unexpectedly."""

def logSMTPTransaction(host, port, transaction):
    server = SocketServer.TCPServer((host, port), xSMTPRequestHandler)
    try:
        pid = os.fork()
        if pid == 0:
            try:
                server.handle_request() # the actual transaction
                server.handle_request() # us dumping it
            finally:
                server.server_close()
                os._exit(0)

        result = transaction()

        client = xSMTPClient(host, port)
        log = client.dump()
        client.quit()

        status = os.waitpid(pid, 0)[1]
        if not os.WIFEXITED(status) or os.WEXITSTATUS(status) != 0:
            raise ChildDiedError

        return (result, log)

    finally:
        server.server_close()

if __name__ == "__main__":
    def transaction():
        client = xSMTPClient("localhost", 8025)
        client.sendmail(
            "gary@localhost", ("root@localhost"), "Hello I'm a message")
        client.quit()
        return 5
        
    print dumpSMTPTransaction("localhost", 8025, transaction)
