import log
import os

class ExecError(Exception):
    """A command exited abnormally"""

def _check_status(cmd, status):
    if status is None:
        return
    if os.WIFEXITED(status):
        status = os.WEXITSTATUS(status)
        if status != 0:
            raise ExecError, "%s exited with code %d" % (cmd, status)
    else:
        raise ExecError, "%s terminated with status %d" % (cmd, status)

def system(*cmd):
    """Execute commands.  Like os.execv(), except it throws exceptions."""
    log.debug('executing "%s"' % " ".join(cmd))

    pid = os.fork()
    if pid == 0:
        os.execv(cmd[0], cmd)
    _check_status(cmd[0], os.waitpid(pid, 0)[1])
    
class CheckingPipe:
    def __init__(self, cmd, pipe):
        self.cmd = cmd
        self.pipe = pipe

    def __getattr__(self, attr):
        return getattr(self.pipe, attr)

    def close(self):
        _check_status(self.cmd, self.pipe.close())

def popen(cmd, mode = "r"):
    """Execute commands.  Like os.popen(), except it throws exceptions."""
    log.debug('executing "%s"' % cmd)
    return CheckingPipe(cmd.split()[0], os.popen(cmd, mode))

def makeAll(basedir):
    """Execute all makefiles beneath a given directory, depth first."""
    dirs = []
    os.path.walk(basedir, lambda out, dir, files: out.extend(
        [dir for file in files if file == "Makefile"]),
                 dirs)

    def cmpfunc(a, b):
        if a.startswith(b):
            return -1
        elif b.startswith(a):
            return 1
        else:
            return cmp(a, b)
    dirs.sort(cmpfunc)

    for dir in dirs:
        if os.environ.has_key("INSTDIR"):
            x = basedir + os.sep
            y = os.environ["INSTDIR"] + os.sep
            assert dir.startswith(x)
            instdir = y + dir[len(x):]
        else:
            instdir = dir

        pipe = popen("/usr/bin/make DIR=%s -C %s 2>&1" % (instdir, dir), "r")
        for line in pipe.xreadlines():
            log.debug(line.rstrip())
        pipe.close()
