# Tomcat Administration Application test
# Coverage: Authentication and Red Hat specifics only.
# Exercises: JSPs, Tag libraries, Struts, Form-based authentication,
#   Role-based authentication, File-based realms, Sessions, Cookies.

from harness.testsuite import TestCase

import os
import re
import urlparse
import xml.sax.handler

class AdminTestCase(TestCase):
    webapp = "admin"

    # Check that authentication works as expected
    def testAuthentication(self):
        class ConfigXmlParser(xml.sax.handler.ContentHandler):
            def startElement(self, tag, attrs):
                if tag == "Context":
                    self.docBase = attrs["docBase"]
        handler = ConfigXmlParser()
        parser = xml.sax.make_parser()
        parser.setContentHandler(handler)
        parser.parse(os.path.join(self.getAppsDir(), self.webapp + ".xml"))

        for url in os.listdir(handler.docBase):
            if not url.endswith(".jsp") or url in ("login.jsp", "error.jsp"):
                continue
            for (user, password, status) in (
                (self.server.adminuser + "x", self.server.adminpass, 401),
                (self.server.adminuser, self.server.adminpass + "x", 401),
                (self.server.manageruser, self.server.managerpass, 403),
                (self.server.adminuser, self.server.adminpass, 200)):
                for withCookies in (1, 0):
                    s = self.getSession()
                    baseurl = s.url

                    r = s.GET(url)
                    self.assertEqual(r.status, 302)
                    location = urlparse.urlunparse(
                        (urlparse.urlparse(baseurl)[0],) +
                        (urlparse.urlparse(r.headers["Location"])[1:]))
                    m = re.search(
                        r"^%slogin.jsp;jsessionid=([0-9a-fA-F]{32})$" % \
                        baseurl, location)
                    self.failUnless(m)
                    session = m.group(1)
                    self.failUnless(s.cookiesMatch(("JSESSIONID", session)))

                    if not withCookies:
                        s.cookies = []
                    r = s.GET("login.jsp;jsessionid=%s" % session)
                    self.assertEqual(r.status, 200)
                    self.assertEqual(r.type, "text/html")
                    formurl = "j_security_check"
                    if not withCookies:
                        formurl += ";jsessionid=%s" % session
                    self.failUnless(re.search(
                        r'^<form method="POST" action=%s\s+'%"'%s'" % formurl+
                        r'^ name="loginForm">', r.body, re.M))

                    r = s.POST(
                        formurl, "j_username=%s&j_password=%s" % (
                            user, password))
                    self.assertEqual(r.status, 302)
                    location = urlparse.urlunparse(
                        (urlparse.urlparse(baseurl)[0],) +
                        (urlparse.urlparse(r.headers["Location"])[1:]))
                    expect = baseurl + (status == 401 and "error.jsp" or url)
                    if not withCookies:
                        expect += ";jsessionid=%s" % session
                    self.assertEqual(location, expect)

                    if status != 401 and url not in (
                        "dumpRegistry.jsp",
                        "dumpServer.jsp",
                        "tree-control-test.jsp"):
                        checkurl = url
                        if not withCookies:
                            checkurl += ";jsessionid=%s" % session
                        r = s.GET(checkurl)
                        self.assertEqual(r.status, status)

    # Check that the writable webapps detection thing works
    def testWritableDirCheck(self):
        s = self.getSession()
        basehref = r'<base href="%s%%s\.jsp">' % urlparse.urlunparse(
            ("http",) + (urlparse.urlparse(s.url)[1:]))
        r = s.GET("checkWritable.do")
        r = s.POST("j_security_check", "j_username=%s&j_password=%s" % (
                self.server.adminuser, self.server.adminpass))
        self.assertEqual(r.status, 302)
        self.failUnless(r.headers["Location"].endswith("/checkWritable.do"))

        dir = os.path.realpath(os.path.join(self.server.dir, "conf"))
        writable_re = re.compile(basehref % "blank")
        unwritable_re = re.compile(basehref % "unwritable")

        oldmode = os.stat(dir).st_mode
        os.chmod(dir, 0555)
        try:
            r = s.GET("checkWritable.do")
            self.assertEqual(r.status, 200)
            self.failUnless(unwritable_re.search(r.body))
            self.failIf(writable_re.search(r.body))
        finally:
            os.chmod(dir, oldmode)
            
        r = s.GET("checkWritable.do")
        self.assertEqual(r.status, 200)
        self.failUnless(writable_re.search(r.body))
        self.failIf(unwritable_re.search(r.body))
