import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

public class TestServlet extends HttpServlet {

    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	resp.setContentType("text/plain");
        PrintWriter out = resp.getWriter();
	String action = req.getParameter("action");

	try {
	    // Connect to the database
	    Context initCtx = new InitialContext();
	    Context envCtx = (Context) initCtx.lookup("java:comp/env");
	    DataSource ds = (DataSource) envCtx.lookup("jdbc/TestDB");
	    Connection conn = ds.getConnection();
	    conn.setAutoCommit(true);
	    Statement stmt = null;
	    if (!action.equals("insert") && !action.equals("list"))
		stmt = conn.createStatement();

	    if (action.equals("assert-mysql")) {

		// Check that the database is a MySQL one
		try {
		    stmt.executeQuery("SHOW STATUS").close();
		    out.println("OK");
		}
		catch (SQLException e) {
		    out.println("FAIL");
		    throw e;
		}
	    }
	    else if (action.equals("assert-pgsql")) {

		// Check that the database is a PostgreSQL one
		try {
		    stmt.executeQuery("SHOW ALL").close();
		    out.println("OK");
		}
		catch (SQLException e) {
		    out.println("FAIL");
		    throw e;
		}
	    }
	    else if (action.equals("create-single")) {

		// Create a table using an individual query
		stmt.executeUpdate(
		    "CREATE TABLE books (" +
		    "  id INT NOT NULL," +
		    "  isbn CHAR(13) NOT NULL," +
		    "  title VARCHAR(100) NOT NULL," +
		    "  PRIMARY KEY (id))");

		out.println("OK");
	    }
	    else if (action.equals("create-batch")) {

		// Create a pair of tables using batched queries
		stmt.addBatch(
		    "CREATE TABLE authors (" +
		    "  id INT NOT NULL," +
		    "  name VARCHAR(100) NOT NULL," +
		    "  PRIMARY KEY (id))");
		stmt.addBatch(
		    "CREATE TABLE authors_map (" +
		    "  book INT NOT NULL," +
		    "  author INT NOT NULL)");
		stmt.executeBatch();

		out.println("OK");
	    }
	    else if (action.equals("insert")) {

		// Populate the three tables we've created using
		// prepared statements

		PreparedStatement bstmt = conn.prepareStatement(
		    "INSERT INTO books VALUES (?, ?, ?)");
		PreparedStatement astmt = conn.prepareStatement(
		    "INSERT INTO authors VALUES (?, ?)");
		PreparedStatement mstmt = conn.prepareStatement(
		    "INSERT INTO authors_map VALUES (?, ?)");

		String[] lines = split(req.getParameter("data"), "\n");
		for (int i = 0; i < lines.length; i++) {
		    String[] bits = split(lines[i], " ");
		    int result;

		    if (bits[0].equals("books")) {
			bstmt.setInt(1, Integer.parseInt(bits[1]));
			bstmt.setString(2, bits[2]);
			bstmt.setString(3, join(" ", bits, 3));

			result = bstmt.executeUpdate();
		    }
		    else if (bits[0].equals("authors")) {
			astmt.setInt(1, Integer.parseInt(bits[1]));
			astmt.setString(2, join(" ", bits, 2));

			result = astmt.executeUpdate();
		    }
		    else if (bits[0].equals("authors_map")) {
			mstmt.setInt(1, Integer.parseInt(bits[1]));
			mstmt.setInt(2, Integer.parseInt(bits[2]));

			result = mstmt.executeUpdate();
		    }
		    else {
			out.println("unknown table '" + bits[0] + "'");
			break;
		    }

		    if (result != 1) {
			out.println("FAIL: result = " + result);
			break;
		    }
		}

		bstmt.close();
		astmt.close();
		mstmt.close();

		out.println("OK");
	    }
	    else if (action.equals("list")) {

		// Try a non-trivial select

		Statement bstmt = conn.createStatement();
		Statement astmt = conn.createStatement();
		
		ResultSet brs = bstmt.executeQuery(
		    "SELECT title, isbn FROM books");
		while (brs.next()) {
		    String title = brs.getString(1);
		    String isbn = brs.getString(2);

		    ResultSet ars = astmt.executeQuery(
			"SELECT name FROM books, authors, authors_map" +
			" WHERE book = books.id" +
			"   AND author = authors.id" +
			"   AND title = '" + title + "'");

		    LinkedList authors = new LinkedList();
		    while (ars.next())
			authors.add(ars.getString(1));

		    out.println(isbn + ": " + title + ": " + join(", ",
			(String[]) authors.toArray(new String[0])));
		}
	    }
	    else if (action.equals("update")) {

		// Try an update
		
		int result = stmt.executeUpdate(
		    "UPDATE books SET isbn = '0-349-10331-3'" +
		    " WHERE isbn = '0-349-10839-0'");

		if (result == 1)
		    out.println("OK");
		else
		    out.println("FAIL: result = " + result);
	    }
	    else if (action.equals("delete")) {

		// Try a delete
		
		int result = stmt.executeUpdate(
		    "DELETE FROM books WHERE title LIKE '%Enterprise%'");

		if (result == 1)
		    out.println("OK");
		else
		    out.println("FAIL: result = " + result);
	    }
	    else {
		out.println("unhandled action '" + action + "'");
	    }

	    if (stmt != null)
		stmt.close();
	    conn.close();

	}
	catch (Exception e) {
	    e.printStackTrace(out);
	} 

    }

    public void doPost(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	doGet(req, resp);
    }
    
    private static String[] split(String str, String sep)
    {
	int nbits = 1;
	for (int i = 0; i < str.length(); nbits++) {
	    int next = str.indexOf(sep, i);
	    if (next == -1)
		break;
	    i = next + sep.length();
	}

	String[] result = new String[nbits];
	for (int i = 0, j = 0; j < nbits; j++) {
	    int next = str.indexOf(sep, i);
	    if (next == -1)
		result[j] = str.substring(i);
	    else
		result[j] = str.substring(i, next);
	    i = next + sep.length();
	}

	return result;
    }

    private static String join(String sep, String[] bits)
    {
	return join(sep, bits, 0);
    }

    private static String join(String sep, String[] bits, int beginindex)
    {
	String result = "";
	for (int i = beginindex; i < bits.length; i++) {
	    result += bits[i];
	    if (i + 1 != bits.length)
		result += sep;
	}
	return result;
    }
}
