from harness.testsuite import TestCase

import glob
import os
from harness import util

class LibGCJTestCase(TestCase):

    # This test can be run anytime (it doesn't even need a running
    # server), but we run it right at the start, since a failure here
    # will probably cause failures all over the place.
    order = 0

    # This testcase is for checking issues which affected Tomcat but
    # have been fixed in libgcj.  In checking for these we can spot
    # libgcj regressions caused by, for example, patches getting lost
    # in rpm upgrades, or by us switching to a completly different
    # libgcj.

    def getTestNames(self):
        """Return a list of test names."""
        return map(self.__pathToTestName,
                   glob.glob(os.path.join(self.getDataDir(), "Test*.java")))

    def __pathToTestName(self, path):
        """Map test classes within the datadir to test function names."""
        dir, file = os.path.split(path)
        if dir != self.getDataDir() or not file.endswith(".java"):
            raise ValueError, file
        return file[0].lower() + file[1:-5]

    def __getattr__(self, attr):
        """Route all calls to execute tests."""
        class Executor:
            def __init__(self, testcase, testclass):
                self.testcase = testcase
                self.testclass = testclass
            def __call__(self):
                return self.testcase.runTest(self.testclass)
        return Executor(self, attr[0].upper() + attr[1:])

    def runTest(self, testclass):
        """Execute a test."""
        cwd = os.getcwd()
        try:
            os.chdir(self.getDataDir())
            pipe = util.popen(os.path.join(self.getDataDir(), testclass), "r")
            failures = [line.rstrip() for line in pipe.xreadlines()]
            pipe.close()
            self.failIf(failures, "\n" + "\n".join(failures))

        finally:
            os.chdir(cwd)
