import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

class TestJavaSqlTimestamp {

    // Check that java.sql.Timestamp.valueOf works (originally fixed
    // in gcc-ssa-3.5ssa-0.20030801.43).  Apart from the expected
    // general database access failures, failure of this method will
    // render the Oracle JDBC driver completly unusable.

    public static void main(String[] args) {

	String fulldate = "2003-11-09 12:34:56.7890123456";

	String[] expected = {"2003-11-09 12:34:56 (0, 0)",
			     "IllegalArgumentException",
			     "2003-11-09 12:34:56 (700, 700000000)",
			     "2003-11-09 12:34:56 (780, 780000000)",
			     "2003-11-09 12:34:56 (789, 789000000)",
			     "2003-11-09 12:34:56 (789, 789000000)",
			     "2003-11-09 12:34:56 (789, 789010000)",
			     "2003-11-09 12:34:56 (789, 789012000)",
			     "2003-11-09 12:34:56 (789, 789012300)",
			     "2003-11-09 12:34:56 (789, 789012340)",
			     "2003-11-09 12:34:56 (789, 789012345)",
			     "IllegalArgumentException"};

	int start = fulldate.length() - expected.length + 1;
	Calendar calendar = new GregorianCalendar();
	SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	for (int i = 0; i < expected.length; i++) {
	    String testdate = fulldate.substring(0, start + i);

	    String result;
	    try {
		Timestamp stamp = Timestamp.valueOf(testdate);
		int nanos = stamp.getNanos();
		calendar.setTime(stamp);
		int millis = calendar.get(Calendar.MILLISECOND);

		result = df.format(stamp) + " (" + millis + ", " + nanos + ")";
	    }
	    catch (IllegalArgumentException e) {
		result = "IllegalArgumentException";
	    }

	    if (!result.equals(expected[i]))
		System.out.println(testdate + " => " + result);
	}
    }
}
